<?php

declare(strict_types=1);
namespace Drupal\translation_bliss\Drush\Commands\PotxCommands\Utility;

use Drupal\Core\Database\Connection;
use phpDocumentor\Reflection\Types\Integer;

/**
 * Annotates strings with their translation status.
 */
final class TranslationStatusFilterAndAnnotator {

  const PREFIX_FOR_CUSTOMIZED = 'CUSTOMIZED';
  const PREFIX_FOR_SHIPPED = 'SHIPPED';
  const PREFIX_FOR_UNTRANSLATED = 'UNTRANSLATED';

  public function __construct(
    private readonly ?string $langcode,
    private readonly bool $filterShipped,
    private ?Connection $connection = NULL,
  ) {
    $this->connection = $this->connection ?? \Drupal::database();
  }

  /**
   * @see \_potx_save_string
   */
  public function _potx_save_string(
    ?string $value = NULL,
    ?int $context = NULL,
    ?string $file = NULL,
    int $line = 0,
    int $string_mode = POTX_STRING_RUNTIME
  ): ?array {
    // Pass through storing strings.
    if (isset($value)) {
      return \_potx_save_string($value, $context, $file, $line, $string_mode);
    }
    // Filter reading of string if requested.
    $strings = \_potx_save_string($value, $context, $file, $line, $string_mode);
    assert(is_array($strings));
    return $this->annotate($strings);
  }

  private function annotate(array $strings): array {
    if (!$this->langcode) {
      return $strings;
    }
    $annotated = [];
    /** @see \_potx_build_files */
    foreach ($strings as $string => $stringInfo) {
      foreach ($stringInfo as $context => $fileInfo) {
        // https://git.drupalcode.org/project/potx/-/merge_requests/24/diffs
        $target = $this->connection->query(
          query: 'SELECT t.translation, t.customized FROM {locales_source} s '
            . 'LEFT JOIN {locales_target} t ON t.lid = s.lid '
            . 'WHERE s.source = :source AND s.context = :context AND t.language = :langcode',
          args: [
            ':source' => $string,
            ':context' => $context,
            ':langcode' => $this->langcode,
          ]
        )->fetchObject();

        $translationStatus = $target->translation ? (
          $target->customized ? self::PREFIX_FOR_CUSTOMIZED : self::PREFIX_FOR_SHIPPED
        ) : self::PREFIX_FOR_UNTRANSLATED;

        $willUseString = $this->filterShipped ?
          ($translationStatus === self::PREFIX_FOR_SHIPPED) :
          ($translationStatus !== self::PREFIX_FOR_SHIPPED);

        // Add annotation by prefixing file info with a dummy file.
        $annotatedFileInfo = [$translationStatus => []] + $fileInfo;
        $annotated[$string][$context] = $annotatedFileInfo;
      }
    }
    return $annotated;
  }

}
